################################################################################
#################### Functions for scorecard colour bars #######################
################################################################################


## modified version of formattable::color_bar() to draw inside a table a colored column bar, also with negative values:
color_bar_neg <- function(color1 = "lightgreen", color2 = "pink", ratio = 1, ...){
  formatter("span",
            style = function(x) style(
              display = "inline-block",
              float = ifelse(x >= 0, "left", "right"),
              "text-align" = ifelse(as.numeric(x) >= 0, "left", "right"),
              "margin-left" = ifelse(as.numeric(x) >= 0, "50%", "0%"),
              "margin-right" = ifelse(as.numeric(x) >= 0,"0%", "50%"),
              "border-radius" = "4px",
              "background-color" = ifelse(as.numeric(x) < 0, color1, color2),
              width = ratio * percent(abs(as.numeric(x)), ...)))
}

## as previous function, but for variables that only have positive values: 
color_bar_pos <- function (color = "lightgray", fun = "proportion_score", score = NULL, ratio = 1, ...){
  fun <- match.fun(fun)
  formatter("span", style = function(x) style(display = "inline-block", float = "left", 
                                              direction = "rtl", "border-radius" = "4px", "padding-right" = "2px", "margin-left" = "25%", 
                                              "background-color" = csscolor(color), width = ratio * percent(fun(as.numeric(x), score))))
}

color_bar_RMSSS <- function (color1 = "violet", color2 = "lawngreen", ratio = 0.7, score = NULL, ...){
  formatter("span", style = function(x) style(display = "inline-block", float = "left", 
                                              direction = "rtl", "border-radius" = "4px", `padding-right` = "2px", "margin-left" = "25%", 
                                              "background-color" = ifelse(x < 1, color1, color2), width = ratio * percent(proportion_score(as.numeric(x), score))))
}

## function to normalize width of the colored column bars in the figure:
proportion_score <- function(x, score = NULL, na.rm = FALSE){
  x / max(abs(score), na.rm = na.rm)
}

## modified version of formattable::color_bar() to draw in a table colored column bars, also with negative values
## it also normalize the values considering the max value of data in the "score" variable (usually shown in the other columns)
color_bar_score <- function(color1 = "lightgreen", color2 = "pink", score = NULL, ratio = 1, ...){
  formatter("span",
            style = function(x) style(
              display = "inline-block",
              float = ifelse(as.numeric(x) >= 0, "left", "right"),
              "text-align" = ifelse(as.numeric(x) >= 0, "left", "right"),
              "margin-left" = ifelse(as.numeric(x) >= 0, "50%", "0%"),
              "margin-right" = ifelse(as.numeric(x) >= 0,"0%", "50%"),
              "border-radius" = "4px",
              "background-color" = ifelse(as.numeric(x) < 0, color1, color2),
              width = ratio * percent(proportion_score(abs(as.numeric(x)), score), ...)))
}