################################################################################
########## Functions for scorecard visualization for type 1 and 3 ##############
################################################################################

create_scorecard_table <- function(transformed_metrics){

    var.name <- variable-dictionary$vars[[recipe$Analysis$Variables$name]]$long_name
    var.unit <- variable-dictionary$vars[[recipe$Analysis$Variables$name]]$units ## 'K'
    inverse.scale <- initialfunction_colorscale(var.name)
    mod.pressure <- ""
    mod.pressure.title <- ""
    system.name <- archive$System[[recipe$Analysis$Datasets$System$name]]$system_name ## 'ECMWF System 5
    reference.name <- archive$Reference[[recipe$Analysis$Datasets$Reference$name]]$reference_name ## 'ERA5'
    start.year <- recipe$Analysis$Time$hcst_start ## 1993
    end.year <- recipe$Analysis$Time$hcst_end ## 2016

    ## Table row titles
    region.names.figure2 <- rep("", n.regions * n.for.months)
    if(n.regions == 3) region.names2 <- region.names else region.names2 <- region.names[-1] 
    for(rr in 1:n.regions) region.names.figure2[floor(n.for.months/2) + (rr-1) * n.for.months ] = region.names2[rr]
    for(rr in 1:n.regions) region.names.figure2[floor(n.for.months/2) + (rr-1) * n.for.months + 1] = region.names.figure[rr]
    forecast.month.figure <- rep(1:n.for.months, n.regions)
    
    ## Define data inside the scorecards table
    table <- data.frame(Region = region.names.figure2, Forecast_month = forecast.month.figure,
                        BiasTarget1 = as.vector(t(mean.bias[1,,])), BiasTarget2 = as.vector(t(mean.bias[2,,])),
                        BiasTarget3 = as.vector(t(mean.bias[3,,])), BiasTarget4 = as.vector(t(mean.bias[4,,])),
                        BiasTarget5 = as.vector(t(mean.bias[5,,])), BiasTarget6 = as.vector(t(mean.bias[6,,])),
                        BiasTarget7 = as.vector(t(mean.bias[7,,])), BiasTarget8 = as.vector(t(mean.bias[8,,])),
                        BiasTarget9 = as.vector(t(mean.bias[9,,])), BiasTarget10 = as.vector(t(mean.bias[10,,])),
                        BiasTarget11 = as.vector(t(mean.bias[11,,])), BiasTarget12 = as.vector(t(mean.bias[12,,])),
                        CorrTarget1 = as.vector(t(mean.corr[1,,])), CorrTarget2 = as.vector(t(mean.corr[2,,])),
                        CorrTarget3 = as.vector(t(mean.corr[3,,])), CorrTarget4 = as.vector(t(mean.corr[4,,])),
                        CorrTarget5 = as.vector(t(mean.corr[5,,])), CorrTarget6 = as.vector(t(mean.corr[6,,])),
                        CorrTarget7 = as.vector(t(mean.corr[7,,])), CorrTarget8 = as.vector(t(mean.corr[8,,])),
                        CorrTarget9 = as.vector(t(mean.corr[9,,])), CorrTarget10 = as.vector(t(mean.corr[10,,])),
                        CorrTarget11 = as.vector(t(mean.corr[11,,])), CorrTarget12 = as.vector(t(mean.corr[12,,])),
                        CrpssTarget1 = as.vector(t(mean.crpss[1,,])), CrpssTarget2 = as.vector(t(mean.crpss[2,,])),
                        CrpssTarget3 = as.vector(t(mean.crpss[3,,])), CrpssTarget4 = as.vector(t(mean.crpss[4,,])),
                        CrpssTarget5 = as.vector(t(mean.crpss[5,,])), CrpssTarget6 = as.vector(t(mean.crpss[6,,])),
                        CrpssTarget7 = as.vector(t(mean.crpss[7,,])), CrpssTarget8 = as.vector(t(mean.crpss[8,,])),
                        CrpssTarget9 = as.vector(t(mean.crpss[9,,])), CrpssTarget10 = as.vector(t(mean.crpss[10,,])),
                        CrpssTarget11 = as.vector(t(mean.crpss[11,,])), CrpssTarget12 = as.vector(t(mean.crpss[12,,])),
                        RpssTarget1 = as.vector(t(mean.rpss[1,,])), RpssTarget2 = as.vector(t(mean.rpss[2,,])),
                        RpssTarget3 = as.vector(t(mean.rpss[3,,])), RpssTarget4 = as.vector(t(mean.rpss[4,,])),
                        RpssTarget5 = as.vector(t(mean.rpss[5,,])), RpssTarget6 = as.vector(t(mean.rpss[6,,])),
                        RpssTarget7 = as.vector(t(mean.rpss[7,,])), RpssTarget8 = as.vector(t(mean.rpss[8,,])),
                        RpssTarget9 = as.vector(t(mean.rpss[9,,])), RpssTarget10 = as.vector(t(mean.rpss[10,,])),
                        RpssTarget11 = as.vector(t(mean.rpss[11,,])), RpssTarget12 = as.vector(t(mean.rpss[12,,])))
    
    ## Cell colors of Bias metric
    bias.max <- max(abs(mean.bias), na.rm = TRUE)
    bias.minmax_max <- quantile(mean.bias, 0.98, na.rm = TRUE)
    bias.minmax_min <- quantile(mean.bias, 0.02, na.rm = TRUE)
    bias.max <- max(abs(bias.minmax_min), abs(bias.minmax_max))
    bias.int <- bias.max * c(-1, -0.8, -0.6, -0.4, -0.2, 0, 0.2, 0.4, 0.6, 0.8, 1) ## one point more than the colors below (the intervals)
    bias.int <- round(bias.int, n.dec)
    
    if(inverse.scale==TRUE){
      bias.palette <- rev(c('#2D004B', '#542789', '#8073AC', '#B2ABD2', '#D8DAEB', '#FEE0B6', '#FDB863', '#E08214', '#B35806', '#7F3B08'))
      colorunder_bias <- "#730C04"
      colorsup_bias <- "#04040E"
    }else{    
      bias.palette <- c('#2D004B', '#542789', '#8073AC', '#B2ABD2', '#D8DAEB', '#FEE0B6', '#FDB863', '#E08214', '#B35806', '#7F3B08')
      colorunder_bias <- "#04040E"
      colorsup_bias <- "#730C04"
    }
    colorunder <- "#04040E" 
    colorsup <- "#730C04"
    
    bias.col <- array(colorunder_bias, c(n.for.months * n.regions, n.start.months)) ## gray is to detect outliers
    bias.text.col <- array("#2A2A2A", c(n.for.months * n.regions, n.start.months)) ## black is the default text color
    bias.text.bold <- array(TRUE, c(n.for.months * n.regions, n.start.months)) ## TRUE is the default bold value
    
    for(mm in 1:n.start.months){
      for(ff in 1:n.for.months){
        for(rr in 1:n.regions){
          for(pp in 1:(length(bias.int)-1)){
            if(is.nan(mean.bias[mm, rr, ff])){
              bias.col[ff + (rr-1) * n.for.months, mm] <- "gray"
            } else {
              if(mean.bias[mm, rr, ff] >= bias.int[pp] && mean.bias[mm, rr, ff] <= bias.int[pp+1])
                bias.col[ff + (rr-1) * n.for.months, mm] <- bias.palette[pp]
            }
            
            ## color text in white and bold if background is gray or dark blue or dark red
            if(is.nan(mean.bias[mm, rr, ff]) ||
               (!is.nan(mean.bias[mm, rr, ff]) && pp == 1 && mean.bias[mm, rr, ff] < bias.int[2]) ||
               (!is.nan(mean.bias[mm, rr, ff]) && pp == 2 && mean.bias[mm, rr, ff] < bias.int[3]) ||
               (!is.nan(mean.bias[mm, rr, ff]) && pp == (length(bias.int)-1) && mean.bias[mm, rr, ff] >= bias.int[length(bias.int)-1])){
              bias.text.col[ff + (rr-1) * n.for.months, mm] <- "white"
            }
          }
          if(mean.bias[mm, rr, ff] < bias.int[1]){bias.col[ff + (rr-1) * n.for.months, mm] <- colorunder_bias}
          if(mean.bias[mm, rr, ff] > bias.int[length(bias.int)]){bias.col[ff + (rr-1) * n.for.months, mm] <- colorsup_bias}
        }
      }
    } 
    
    ## cell colors of Correlation metric
    Corr.int <- c(-1, -0.8, -0.6, -0.4, -0.2, 0, 0.2, 0.4, 0.6, 0.8, 1) ## one point more than the colors below (the intervals)
    Corr.palette <- c('#2D004B', '#542789', '#8073AC', '#B2ABD2', '#D8DAEB', '#FEE0B6', '#FDB863', '#E08214', '#B35806', '#7F3B08')
    Corr.col <- array("gray", c(n.for.months * n.regions, n.start.months)) ## gray is to detect outliers
    Corr.text.col <- array("#2A2A2A", c(n.for.months * n.regions, n.start.months)) ## black is the default text color
    Corr.text.bold <- array(TRUE, c(n.for.months * n.regions, n.start.months)) ## TRUE is the default bold value
    
    for(mm in 1:n.start.months){
      for(ff in 1:n.for.months){
        for(rr in 1:n.regions){
          for(pp in 1:(length(Corr.int)-1)){
            if(is.nan(mean.corr[mm, rr, ff])){
              Corr.col[ff + (rr-1) * n.for.months, mm] <- "gray"
            } else {
              if(mean.corr[mm, rr, ff] >= Corr.int[pp] && mean.corr[mm, rr, ff] <= Corr.int[pp+1])
                Corr.col[ff + (rr-1) * n.for.months, mm] <- Corr.palette[pp]
            }
            
            ## color text in white and bold if background is gray or dark blue or dark red
            if(is.nan(mean.corr[mm, rr, ff]) ||
               (!is.nan(mean.corr[mm, rr, ff]) && pp == 1 && mean.corr[mm, rr, ff] < Corr.int[2]) ||
               (!is.nan(mean.corr[mm, rr, ff]) && pp == 2 && mean.corr[mm, rr, ff] < Corr.int[3]) ||
               (!is.nan(mean.corr[mm, rr, ff]) && pp == (length(Corr.int)-1) && mean.corr[mm, rr, ff] >= Corr.int[length(Corr.int)-1])){
              Corr.text.col[ff + (rr-1) * n.for.months, mm] <- "white"
              Corr.text.bold[ff + (rr-1) * n.for.months, mm] <- TRUE
            }
          }
        }
      }
    } 
    
    ## cell colors of CRPSS metric
    CRPSS.int <- c(-0.5, -0.4, -0.3, -0.2, -0.1, 0, 0.1, 0.2, 0.3, 0.4, 0.5) ## one point more than the colors below (the intervals)
    CRPSS.palette <- c('#2D004B', '#542789', '#8073AC', '#B2ABD2', '#D8DAEB', '#FEE0B6', '#FDB863', '#E08214', '#B35806', '#7F3B08') #palette as suggested in meeting
    CRPSS.col <- array(colorunder, c(n.for.months * n.regions, n.start.months)) ## blue is to detect outliers
    CRPSS.text.col <- array("#2A2A2A", c(n.for.months * n.regions, n.start.months)) ## black is the default text color
    CRPSS.text.bold <- array(TRUE, c(n.for.months * n.regions, n.start.months)) ## TRUE is the default bold value
    
    for(mm in 1:n.start.months){
      for(ff in 1:n.for.months){
        for(rr in 1:n.regions){
          for(pp in 1:(length(CRPSS.int)-1)){
            if(is.nan(mean.crpss[mm, rr, ff])){
              CRPSS.col[ff + (rr-1) * n.for.months, mm] <- "gray"
            } else {
              if(mean.crpss[mm, rr, ff] < CRPSS.int[1])
                CRPSS.col[ff + (rr-1) * n.for.months, mm] <- colorunder
              if(mean.crpss[mm, rr, ff] >= CRPSS.int[length(CRPSS.int)-1])
                CRPSS.col[ff + (rr-1) * n.for.months, mm] <- colorsup
              
              if(mean.crpss[mm, rr, ff] >= CRPSS.int[pp] && mean.crpss[mm, rr, ff] <= CRPSS.int[pp+1])
                CRPSS.col[ff + (rr-1) * n.for.months, mm] <- CRPSS.palette[pp]
            }
            
            ## color text in white and bold if background is gray or dark blue or dark red
            if(is.nan(mean.crpss[mm, rr, ff]) ||
               (!is.nan(mean.crpss[mm, rr, ff]) && pp == 1 && mean.crpss[mm, rr, ff] < CRPSS.int[2]) ||
               (!is.nan(mean.crpss[mm, rr, ff]) && pp == 2 && mean.crpss[mm, rr, ff] < CRPSS.int[3]) ||
               (!is.nan(mean.crpss[mm, rr, ff]) && pp == (length(CRPSS.int)-1) && mean.crpss[mm, rr, ff] >= CRPSS.int[length(CRPSS.int)-1])){
              CRPSS.text.col[ff + (rr-1) * n.for.months, mm] <- "white"
              CRPSS.text.bold[ff + (rr-1) * n.for.months, mm] <- TRUE
            }
          }
        }
      }
    }
    
    ## cell colors of RPSS metric
    RPSS.int <- CRPSS.int
    RPSS.palette <- CRPSS.palette
    RPSS.col <- array(colorunder, c(n.for.months * n.regions, n.start.months)) ## "#045a8d" blue is to detect outliers
    RPSS.text.col <- array("#2A2A2A", c(n.for.months * n.regions, n.start.months)) ## black is the default text color
    RPSS.text.bold <- array(TRUE, c(n.for.months * n.regions, n.start.months)) ## TRUE is the default bold value
    
    for(mm in 1:n.start.months){
      for(ff in 1:n.for.months){
        for(rr in 1:n.regions){
          for(pp in 1:(length(RPSS.int)-1)){
            if(is.nan(mean.rpss[mm, rr, ff])){
              RPSS.col[ff + (rr-1) * n.for.months, mm] <- "gray"
            } else {
              if(mean.rpss[mm, rr, ff] < RPSS.int[1])
                RPSS.col[ff + (rr-1) * n.for.months, mm] <- colorunder
              if(mean.rpss[mm, rr, ff] >= RPSS.int[length(RPSS.int)-1])
                RPSS.col[ff + (rr-1) * n.for.months, mm] <- colorsup
              
              if(mean.rpss[mm, rr, ff] >= RPSS.int[pp] && mean.rpss[mm, rr, ff] <= RPSS.int[pp+1])
                RPSS.col[ff + (rr-1) * n.for.months, mm] <- RPSS.palette[pp]
            }
            
            ## color text in white if background is gray or deep blue:
            if(is.nan(mean.rpss[mm, rr, ff]) ||
               (!is.nan(mean.rpss[mm, rr, ff]) && pp == 1 && mean.rpss[mm, rr, ff] < RPSS.int[2]) ||
               (!is.nan(mean.rpss[mm, rr, ff]) && pp == 2 && mean.rpss[mm, rr, ff] < RPSS.int[3]) ||
               (!is.nan(mean.rpss[mm, rr, ff]) && pp == (length(RPSS.int)-1) && mean.rpss[mm, rr, ff] >= RPSS.int[length(RPSS.int)-1])){
              RPSS.text.col[ff + (rr-1) * n.for.months, mm] <- "white"
              RPSS.text.bold[ff + (rr-1) * n.for.months, mm] <- TRUE
            }
          }
        }
      }
    }
    
    ## Set table headers
    my.col.names <- c("Region", "Forecast month", rep(c("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"), 4))
    
    ## Add capital to first letter and remove the capitals in the second part of the variable name:
    var.name.min <- paste0(toupper(substring(var.name, 1, 1)), substring(str_to_lower(var.name), 2, nchar(var.name)))
    
    
    options(stringsAsFactors = FALSE)
    my.header.names <- as.data.frame(data.frame(c1=c("", paste0("Mean Bias (", var.unit,")"), "Correlation", "CRPSS", "RPSS"), c2=c(2, rep(n.start.months, 4))))
    my.header.names2 <- data.frame(c1=paste0(var.name.min, mod.pressure.title, " of ", system.name), c2=50)
    ref.label <- data.frame(c1=paste0("(Ref: ", reference.name, " ", start.year, "-", end.year, ")"), c2=50)
    
    
    ## remove "known issue" data from cmcc-3:
    ## remove corrupted start dates of CMCC S3:
    if(system.name == "CMCC-S3"){
      if(plevel==""){
        ## case of single-level variables:
        for(j in c(5, 9, 17, 21, 29, 33, 41, 45)) table[,j] <- NaN
        bias.col[, c(3,7)] <- Corr.col[, c(3,7)] <- CRPSS.col[, c(3,7)] <- RPSS.col[, c(3,7)] <- "gray"
        bias.text.col[, c(3, 7)] <- Corr.text.col[, c(3,7)] <- CRPSS.text.col[, c(3,7)] <- RPSS.text.col[, c(3,7)] <- "white"
      } else { ## case of pressure-levels variables:
        for(j in c(5:11, 17:23, 29:35, 41:47)) table[,j] <- NaN
        bias.col[, 3:9] <- Corr.col[, 3:9] <- CRPSS.col[, 3:9] <- RPSS.col[, 3:9] <- "gray"
        bias.text.col[, 3:9] <- Corr.text.col[, 3:9] <- CRPSS.text.col[, 3:9] <- RPSS.text.col[, 3:9] <- "white"
      }
    }
    
    ## as above, but without normalization:
    table2 <- table
    
    ## Select the number of decimals to show, their font size and color
    for(col in 3:(2 + n.start.months * 4)) {
      if (col > (2 + n.start.months)) table2[,col] <- format(table[,col], nsmall = 2) else table2[,col] <- format(table[,col], nsmall = n.dec) 
      table2[,col] <- cell_spec(table2[,col], font_size = 8, color="gray70") ## smaller font size
    }
    
    n.columns <- 2 + n.start.months * 4
    
    ## Replace with "No Data" cells with NaN
    for(i in 3:n.columns) table2[,i] <- str_replace(table2[,i], "NaN", "")
    
    ## Change the font size of any value in the table lower than -1000 with a smaller one, to fit in its cell: (this usually happens with CRPSS)
    for(i in 3:n.columns) { ss <- which(table[,i] < -1000) ; if(length(ss) > 0) table2[ss,i] <- str_replace(table2[ss,i], "8px", "5px")}
    
    ##### Create html table  #####
    table.html <- kbl(table2, escape = F, col.names=my.col.names, align = rep("c", n.columns)) %>%
      kable_paper("hover", full_width = F) %>%
      add_header_above(c(" " = 2, table_column_header = n.start.months, table_column_header = n.start.months, table_column_header = n.start.months, table_column_header = n.start.months)) %>%
      add_header_above(header = ref.label, align = "left", font_size = 14) %>% 
      add_header_above(header = my.header.names, font_size = 20) %>%
      add_header_above(header = my.header.names2, font_size = 20, align = "left") %>%
      
      column_spec(2 + n.start.months*0 + 1, background = bias.col[,1], color = bias.text.col[,1], bold = bias.text.bold[,1]) %>%
      column_spec(2 + n.start.months*0 + 2, background = bias.col[,2], color = bias.text.col[,2], bold = bias.text.bold[,2]) %>%
      column_spec(2 + n.start.months*0 + 3, background = bias.col[,3], color = bias.text.col[,3], bold = bias.text.bold[,3]) %>%
      column_spec(2 + n.start.months*0 + 4, background = bias.col[,4], color = bias.text.col[,4], bold = bias.text.bold[,4]) %>%
      column_spec(2 + n.start.months*0 + 5, background = bias.col[,5], color = bias.text.col[,5], bold = bias.text.bold[,5]) %>%
      column_spec(2 + n.start.months*0 + 6, background = bias.col[,6], color = bias.text.col[,6], bold = bias.text.bold[,6]) %>%
      column_spec(2 + n.start.months*0 + 7, background = bias.col[,7], color = bias.text.col[,7], bold = bias.text.bold[,7]) %>%
      column_spec(2 + n.start.months*0 + 8, background = bias.col[,8], color = bias.text.col[,8], bold = bias.text.bold[,8]) %>%
      column_spec(2 + n.start.months*0 + 9, background = bias.col[,9], color = bias.text.col[,9], bold = bias.text.bold[,9]) %>%
      column_spec(2 + n.start.months*0 + 10, background = bias.col[,10], color = bias.text.col[,10], bold = bias.text.bold[,10]) %>%
      column_spec(2 + n.start.months*0 + 11, background = bias.col[,11], color = bias.text.col[,11], bold = bias.text.bold[,11]) %>%
      column_spec(2 + n.start.months*0 + 12, background = bias.col[,12], color = bias.text.col[,12], bold = bias.text.bold[,12]) %>%
      
      column_spec(2 + n.start.months*1 + 1, background = Corr.col[,1], color = Corr.text.col[,1], bold = Corr.text.bold[,1]) %>%
      column_spec(2 + n.start.months*1 + 2, background = Corr.col[,2], color = Corr.text.col[,2], bold = Corr.text.bold[,2]) %>%
      column_spec(2 + n.start.months*1 + 3, background = Corr.col[,3], color = Corr.text.col[,3], bold = Corr.text.bold[,3]) %>%
      column_spec(2 + n.start.months*1 + 4, background = Corr.col[,4], color = Corr.text.col[,4], bold = Corr.text.bold[,4]) %>%
      column_spec(2 + n.start.months*1 + 5, background = Corr.col[,5], color = Corr.text.col[,5], bold = Corr.text.bold[,5]) %>%
      column_spec(2 + n.start.months*1 + 6, background = Corr.col[,6], color = Corr.text.col[,6], bold = Corr.text.bold[,6]) %>%
      column_spec(2 + n.start.months*1 + 7, background = Corr.col[,7], color = Corr.text.col[,7], bold = Corr.text.bold[,7]) %>%
      column_spec(2 + n.start.months*1 + 8, background = Corr.col[,8], color = Corr.text.col[,8], bold = Corr.text.bold[,8]) %>%
      column_spec(2 + n.start.months*1 + 9, background = Corr.col[,9], color = Corr.text.col[,9], bold = Corr.text.bold[,9]) %>%
      column_spec(2 + n.start.months*1 + 10, background = Corr.col[,10], color = Corr.text.col[,10], bold = Corr.text.bold[,10]) %>%
      column_spec(2 + n.start.months*1 + 11, background = Corr.col[,11], color = Corr.text.col[,11], bold = Corr.text.bold[,11]) %>%
      column_spec(2 + n.start.months*1 + 12, background = Corr.col[,12], color = Corr.text.col[,12], bold = Corr.text.bold[,12]) %>%
      
      column_spec(2 + n.start.months*2 + 1, background = CRPSS.col[,1], color = CRPSS.text.col[,1], bold = CRPSS.text.bold[,1]) %>%
      column_spec(2 + n.start.months*2 + 2, background = CRPSS.col[,2], color = CRPSS.text.col[,2], bold = CRPSS.text.bold[,2]) %>%
      column_spec(2 + n.start.months*2 + 3, background = CRPSS.col[,3], color = CRPSS.text.col[,3], bold = CRPSS.text.bold[,3]) %>%
      column_spec(2 + n.start.months*2 + 4, background = CRPSS.col[,4], color = CRPSS.text.col[,4], bold = CRPSS.text.bold[,4]) %>%
      column_spec(2 + n.start.months*2 + 5, background = CRPSS.col[,5], color = CRPSS.text.col[,5], bold = CRPSS.text.bold[,5]) %>%
      column_spec(2 + n.start.months*2 + 6, background = CRPSS.col[,6], color = CRPSS.text.col[,6], bold = CRPSS.text.bold[,6]) %>%
      column_spec(2 + n.start.months*2 + 7, background = CRPSS.col[,7], color = CRPSS.text.col[,7], bold = CRPSS.text.bold[,7]) %>%
      column_spec(2 + n.start.months*2 + 8, background = CRPSS.col[,8], color = CRPSS.text.col[,8], bold = CRPSS.text.bold[,8]) %>%
      column_spec(2 + n.start.months*2 + 9, background = CRPSS.col[,9], color = CRPSS.text.col[,9], bold = CRPSS.text.bold[,9]) %>%
      column_spec(2 + n.start.months*2 + 10, background = CRPSS.col[,10], color = CRPSS.text.col[,10], bold = CRPSS.text.bold[,10]) %>%
      column_spec(2 + n.start.months*2 + 11, background = CRPSS.col[,11], color = CRPSS.text.col[,11], bold = CRPSS.text.bold[,11]) %>%
      column_spec(2 + n.start.months*2 + 12, background = CRPSS.col[,12], color = CRPSS.text.col[,12], bold = CRPSS.text.bold[,12]) %>%
      
      column_spec(2 + n.start.months*3 + 1, background = RPSS.col[,1], color = RPSS.text.col[,1], bold = RPSS.text.bold[,1]) %>%
      column_spec(2 + n.start.months*3 + 2, background = RPSS.col[,2], color = RPSS.text.col[,2], bold = RPSS.text.bold[,2]) %>%
      column_spec(2 + n.start.months*3 + 3, background = RPSS.col[,3], color = RPSS.text.col[,3], bold = RPSS.text.bold[,3]) %>%
      column_spec(2 + n.start.months*3 + 4, background = RPSS.col[,4], color = RPSS.text.col[,4], bold = RPSS.text.bold[,4]) %>%
      column_spec(2 + n.start.months*3 + 5, background = RPSS.col[,5], color = RPSS.text.col[,5], bold = RPSS.text.bold[,5]) %>%
      column_spec(2 + n.start.months*3 + 6, background = RPSS.col[,6], color = RPSS.text.col[,6], bold = RPSS.text.bold[,6]) %>%
      column_spec(2 + n.start.months*3 + 7, background = RPSS.col[,7], color = RPSS.text.col[,7], bold = RPSS.text.bold[,7]) %>%
      column_spec(2 + n.start.months*3 + 8, background = RPSS.col[,8], color = RPSS.text.col[,8], bold = RPSS.text.bold[,8]) %>%
      column_spec(2 + n.start.months*3 + 9, background = RPSS.col[,9], color = RPSS.text.col[,9], bold = RPSS.text.bold[,9]) %>%
      column_spec(2 + n.start.months*3 + 10, background = RPSS.col[,10], color = RPSS.text.col[,10], bold = RPSS.text.bold[,10]) %>%
      column_spec(2 + n.start.months*3 + 11, background = RPSS.col[,11], color = RPSS.text.col[,11], bold = RPSS.text.bold[,11]) %>%
      column_spec(2 + n.start.months*3 + 12, background = RPSS.col[,12], color = RPSS.text.col[,12], bold = RPSS.text.bold[,12]) %>%
      
      column_spec(1, bold = TRUE, width_min = "4cm", width_max = "4cm") %>%
      column_spec(2, bold = TRUE, width_max = "1cm") %>%
      column_spec(3:n.columns, width_max = "0.9cm", width_min = "0.9cm") %>%
      column_spec(c(1, 2, 2+n.start.months, 2+n.start.months*2, 2+n.start.months*3), border_right = "1px solid black") %>%
      column_spec(1, border_left = "4px solid gray") %>%
      column_spec(n.columns, border_right = "4px solid gray") %>%
      
      column_spec(c(1,2,14,26,38), border_right = "1px solid black") %>%
      row_spec(seq(from = 0, to = n.for.months * n.regions, by = n.for.months), extra_css = "border-bottom: 2px solid black", hline_after = TRUE) #1px solid")
    
    
    #return(table.html)
    
    ## Create colour bar legends
    png(filename= paste0(fileout, "/scorecard_legend.png"), width=455, height=40)
    ColorBar(brks = CRPSS.int, cols = CRPSS.palette, vertical = FALSE, var_limits=c(-10000000, 1), col_inf = colorunder, col_sup = colorsup) # "#000080")
    dev.off()
    
    png(filename= paste0(fileout, "/scorecard_legend_bias.png"), width=455, height=40)
    ColorBar(brks = round(bias.int, n.dec), cols = bias.palette, vertical = FALSE, triangle_ends_scale = 1, col_inf = colorunder_bias, col_sup = colorsup_bias) 
    dev.off()
    
    png(filename= paste0(fileout, "/scorecard_legend_corr.png"), width=450, height=40)
    ColorBar(brks = Corr.int, cols = Corr.palette, vertical = FALSE, triangle_ends_scale = 0, col_inf = colorunder) 
    dev.off()
    
    currentworkingdir <- getwd()
    setwd(fileout)
    
    ## Add white space to the left of the first colour bar legend
    system(paste0("convert ./scorecard_legend_bias.png -background white -splice 225x0 ./scorecard_legend_bias_full.png"))
    
    ## Paste all legends together
    system(paste0("montage ./scorecard_legend_bias_full.png ./scorecard_legend_corr.png ./scorecard_legend.png ./scorecard_legend.png -tile 4x1 -geometry -4+10 ./legend_scorecard.png"))
    unlink('scorecard_legend*')
    setwd(currentworkingdir)
    
    ## Save html table as png
    my.file <- paste0(fileout, '/scorecard.png')
    try(save_kable(table.html, file = my.file), silent = TRUE)
    
    ## Remove temporal directories (not automatically removed due to error with save_kable)
    unlink(paste0(fileout, './saved_results/lib*'), recursive = TRUE) # clean temporal directories (not automatically removed because of error with save_kable)
    
    ## Add white space to the top of the scorecards, to be cut when adding legend:
    setwd(fileout)
    system(paste0("convert ./scorecard.png -background white -splice 0x20 ./scorecard.png"))
    
    ## Add the legends below the scorecard table
    system(paste0("montage ./scorecard.png ./legend_scorecard.png -tile 1x2 -geometry +0-10 ./scorecard.png"))
    
   # return(scorecard.png)
}




##### Image path to save ######

save_scorecard <- function(system.filename, reference.filename, var.filename, start.year, end.year, mod.pressure, hindcasts.filename, scorecard_type){
  
  if(plevel==""){
    scorecard_save_path <- paste0(fileout, "/", system.filename,
                                  "/scorecard-", scorecard_type, "_", system.filename, "_", reference.filename, "_",
                                  var.filename, "_", hindcasts.filename, ".png")
  }else{
    scorecard_save_path <- paste0(fileout, "/", system.filename,
                                  "/scorecard-", scorecard_type, "_", system.filename, "_", reference.filename, "_",
                                  var.filename, "_", mod.pressure, "_", hindcasts.filename, ".png")
    
  }
  return(scorecard_save_path)
  
}


    